'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require('events');

var _jsLogger = require('js-logger');

var _jsLogger2 = _interopRequireDefault(_jsLogger);

var _justCompose = require('just-compose');

var _justCompose2 = _interopRequireDefault(_justCompose);

var _observable = require('./lib/observable');

var _observable2 = _interopRequireDefault(_observable);

var _flatMap = require('./lib/observable/flatMap');

var _QueuedSocket = require('./QueuedSocket');

var _QueuedSocket2 = _interopRequireDefault(_QueuedSocket);

var _State = require('./State');

var _State2 = _interopRequireDefault(_State);

var _createAccessors = require('./lib/createAccessors');

var _createAccessors2 = _interopRequireDefault(_createAccessors);

var _mapValues = require('./lib/mapValues');

var _constants = require('./constants');

var _constants2 = _interopRequireDefault(_constants);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/** @module */

var SENSEME_PORT = _constants2.default.SENSEME_PORT;


var $private = Symbol();

/**
 * Represents one device on the local network, such as a fan.
 */

var Device = function () {
    /**
     * Create a new device.
     * @param {Object} opts
     * @param {string} opts.name - The advertised name of the new device
     * @param {string} opts.id   - The MAC address of the new device
     * @param {string} opts.type - The type of the new device, such as `FAN,HAIKU,HSERIES`
     * @param {string} opts.ip   - The IP address of the new device
     * @param {EventEmitter} messenger - An EventEmitter for funneling incoming messages from the discovery process.
     */

    function Device(_ref, messenger) {
        var _this = this;

        var name = _ref.name;
        var id = _ref.id;
        var type = _ref.type;
        var ip = _ref.ip;

        _classCallCheck(this, Device);

        _jsLogger2.default.debug('constructing a device: ' + name + ', ' + id + ', ' + type + ', ' + ip);
        this[$private] = { name: name, id: id, type: type, ip: ip };

        var socket = this[$private].socket = new _QueuedSocket2.default(ip, SENSEME_PORT, /\([^)]+\)/g);

        socket.on('data', function (data) {
            return _this._handleMessage(data.toString());
        });
        if (messenger) {
            messenger.on('message', function (msg) {
                return _this._handleMessage(msg.toString());
            });
        }

        if (!name || !id) {
            // we'll need to know both the ID and the name
            // to ensure successful communication with the fan.
            socket.send('<ALL;DEVICE;ID;GET>');
        }

        if (_observable.hasObservable) {
            /**
             * Observe changes to any of the properties of this device.  This method is only
             * available if your application has an Observable library installed.
             * See {@link https://www.npmjs.com/package/any-observable} for details.
             * @memberof module:Device~Device
             * @instance
             * @returns {Observable} An ES7-compatible obsevable that emits objects of the form `{ path, value }` for each property change.
             */
            var observeAll = function observeAll() {
                var _context;

                return (_context = _this.state.observeAll(), _flatMap.method).call(_context, function (x) {
                    return _observable2.default.from((0, _mapValues.translateHardwareValues)(x));
                });
            };

            this.observeAll = observeAll;
        }

        var accessorImplementations = {
            get: function get(path) {
                return (0, _mapValues.mapBack)(path, _this.state.get(path));
            },
            set: function set(path, value) {
                _this._sendMessage([].concat(_toConsumableArray(path), [value]));
            },

            refresh: function refresh(path) {
                _this._sendMessage(path);
            },

            listen: function listen(path) {
                return _this.state.listen(path);
            },
            observe: function observe(path) {
                return _this.state.observe(path);
            }
        };

        this.state = new _State2.default();
        _createAccessors2.default.call(this, accessorImplementations);
    }

    /**
     * Send one or more messages to the device.
     * @param {...string} msgs - The messages to send
     * @private
     */


    _createClass(Device, [{
        key: '_sendMessage',
        value: function _sendMessage() {
            var _this2 = this;

            for (var _len = arguments.length, msgs = Array(_len), _key = 0; _key < _len; _key++) {
                msgs[_key] = arguments[_key];
            }

            this[$private].socket.send(msgs.map(function (x) {
                return Array.isArray(x) ? x.join(';') : x;
            }).map(function (x) {
                return '<' + (_this2.id || _this2.name) + ';' + x + '>';
            }).join(''));
        }

        /**
         * Process one incoming message.
         * @param {string} msg - The message
         * @private
         */

    }, {
        key: '_handleMessage',
        value: function _handleMessage(msg) {
            _jsLogger2.default.debug('Incoming: ' + msg);
            var path = msg.replace(/^\(([^)]+)\)$/, '$1').split(';');
            var deviceName = path.shift();

            if (path[0] === 'DEVICE' && path[1] === 'ID') {
                this[$private].name = deviceName;
                this[$private].type = path.pop();
                this[$private].id = path.pop();
                this.refreshAll();
            } else {
                var newValue = void 0;
                /*
                * a path like SLEEP;EVENT;ON should be parsed as
                *     path = [SLEEP, EVENT, ON]
                *     value = undefined
                *
                * the naive implementation for now is -- if the
                * next-to-last path element is EVENT, assume the
                * value is undefined, otherwise assume the value
                * is the last element.
                */
                if (path[path.length - 2] !== 'EVENT') {
                    newValue = path.pop();
                }

                if (deviceName === this.name) {
                    this.state.set(path, newValue);
                }
            }
        }

        /**
         * Disconnect from the device.
         */

    }, {
        key: 'disconnect',
        value: function disconnect() {
            this[$private].socket.close();
            this[$private].socket = undefined;
        }

        /**
         * Listen for changes to any of the properties of this device.
         * Changes are delivered as 'change' events emitted by the returned
         * emitter.
         * @returns {EventEmitter}
         */

    }, {
        key: 'listenAll',
        value: function listenAll() {
            var ret = new _events.EventEmitter();

            this.state.listenAll().on('change', (0, _justCompose2.default)(_mapValues.translateHardwareValues, function (items) {
                return items.forEach(function (x) {
                    return ret.emit('change', x);
                });
            }));

            return ret;
        }

        /**
         * Request the current values of all known properties from this device.
         * This method only updates the local state; to be informed of the new
         * values, use {@link Device#listenAll}, {@link Device#observeAll}, or the corresponding
         * methods on the individual values of interest.
         */

    }, {
        key: 'refreshAll',
        value: function refreshAll() {
            // for whatever reason, SNSROCC;STATUS is skipped by GETALL.
            this._sendMessage('GETALL', 'SNSROCC;STATUS;GET', 'NAME;VALUE;GET');
        }

        /**
         * The ID (MAC address) of this device.
         * @type {string}
         * @readonly
         */

    }, {
        key: 'id',
        get: function get() {
            return this[$private].id;
        }

        /**
         * The IP address of this device.
         * @type {string}
         * @readonly
         */

    }, {
        key: 'ip',
        get: function get() {
            return this[$private].ip;
        }

        /**
         * The name of this device
         * @type {string}
         * @readonly
         */

    }, {
        key: 'name',
        get: function get() {
            return this[$private].name;
        }

        /**
         * The type of this device, such as `FAN,HAIKU,HSERIES`
         * @type {string}
         * @readonly
         */

    }, {
        key: 'type',
        get: function get() {
            return this[$private].type;
        }
    }]);

    return Device;
}();

/**
 * @name fan
 * @type {FanProperties}
 * @memberof module:Device~Device
 * @readonly
 * @instance
 */

/**
 * @name light
 * @type {LightProperties}
 * @memberof module:Device~Device
 * @readonly
 * @instance
 */

/**
 * @name sensor
 * @type {SensorProperties}
 * @memberof module:Device~Device
 * @readonly
 * @instance
 */

/**
 * @name smartMode
 * @type {SmartModeProperties}
 * @memberof module:Device~Device
 * @readonly
 * @instance
 */

/**
 * @name sleepMode
 * @type {SleepModeProperties}
 * @memberof module:Device~Device
 * @readonly
 * @instance
 */

/**
 * @name device
 * @type {DeviceProperties}
 * @memberof module:Device~Device
 * @readonly
 * @instance
 */

/**
 * @name network
 * @type {NetworkProperties}
 * @memberof module:Device~Device
 * @readonly
 * @instance
 */

exports.default = Device;