'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _net = require('net');

var _stream = require('stream');

var _process = require('process');

var _jsLogger = require('js-logger');

var _jsLogger2 = _interopRequireDefault(_jsLogger);

var _observable = require('./lib/observable');

var _observable2 = _interopRequireDefault(_observable);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var $private = Symbol();

function setUpConnection() {
    var _this = this;

    var socket = new _net.Socket({
        readable: true,
        writable: true
    });
    socket.setEncoding('utf8');

    this[$private].socket = socket;

    var handleClose = function handleClose() {
        _this[$private].socket = undefined;
        _this[$private].connected = undefined;
    };

    var handleConnect = function handleConnect() {
        _jsLogger2.default.info("socket connected; let's do some work");

        _this[$private].connected = true;
        socket.on('data', function (raw) {
            var buffer = _this[$private].buffer + raw;

            var m = void 0,
                re = _this[$private].inputRegex;

            if (!re.flags.includes('g')) {
                re = new RegExp(re.source, re.flags + 'g');
            }

            do {
                var lastIdx = re.lastIndex;

                m = re.exec(buffer);
                if (m) {
                    if (_this[$private].isReading) {
                        _this.push(m[0]);
                    } else {
                        _this[$private].incomingQueue.push(m[0]);
                    }

                    var sub = _this[$private].subject;
                    if (sub && !sub.closed) {
                        sub.next(m[0]);
                    }
                } else {
                    _this[$private].buffer = buffer.substr(lastIdx);
                }
            } while (m);
        });

        (0, _process.nextTick)(function () {
            processQueue.call(_this);
        });
    };

    socket.on('connect', handleConnect).on('close', handleClose);

    socket.connect(this._port, this._ip);
}

function processQueue() {
    var _this2 = this;

    var _$private = this[$private];
    var _$private$outgoingQue = _$private.outgoingQueue;
    var outgoingQueue = _$private$outgoingQue === undefined ? [] : _$private$outgoingQue;
    var alreadyWorking = _$private.alreadyWorking;
    var connected = _$private.connected;
    var socket = _$private.socket;

    // if there's nothing to do, get out.

    if (!outgoingQueue.length) {
        _jsLogger2.default.info('nothing to do!');
        return;
    }

    // if we're already doing something, get out.
    if (alreadyWorking) {
        _jsLogger2.default.info('already working!');
        return;
    }

    // if a connection is pending, wait.
    if (connected === false) {
        _jsLogger2.default.info('connection pending');
        return;
    }

    // if there's no network connection, make one and get out.
    if (connected === undefined) {
        this[$private].connected = false;
        _jsLogger2.default.info("no connection; let's make one.");
        setUpConnection.call(this);
        return;
    }

    _jsLogger2.default.info('Getting down to business');

    // this is probably not really needed.
    this[$private].alreadyWorking = true;

    // if there is something to do and there's a connection, process
    // the first item in the queue
    var cmd = outgoingQueue.shift();
    _jsLogger2.default.debug("sent: " + cmd);
    socket.write(cmd);
    (0, _process.nextTick)(function () {
        processQueue.call(_this2);
    });
    this[$private].alreadyWorking = false;
}

var QueuedSocket = function (_Duplex) {
    _inherits(QueuedSocket, _Duplex);

    function QueuedSocket(ip, port) {
        var inputRegex = arguments.length <= 2 || arguments[2] === undefined ? /.*/g : arguments[2];

        _classCallCheck(this, QueuedSocket);

        var _this3 = _possibleConstructorReturn(this, (QueuedSocket.__proto__ || Object.getPrototypeOf(QueuedSocket)).call(this));

        _this3._ip = ip;
        _this3._port = port;

        _this3[$private] = {
            outgoingQueue: [],
            incomingQueue: [],
            connected: false,
            inputRegex: inputRegex
        };

        _jsLogger2.default.info("no connection; let's make one.");
        setUpConnection.call(_this3);

        if (_observable.hasObservable) {
            (function () {
                var self = _this3;
                var descriptor = {
                    get: function get() {
                        return new _observable2.default(function (observer) {
                            self[$private].subject = observer;

                            return function () {
                                delete self[$private].subject;
                            };
                        });
                    }
                };

                Object.defineProperty(_this3, 'data', descriptor);
            })();
        }
        return _this3;
    }

    _createClass(QueuedSocket, [{
        key: '_read',
        value: function _read(size) {
            var priv = this[$private];

            priv.isReading = true;

            // first empty the queue if there is one
            while (priv.isReading && priv.incomingQueue.length) {
                priv.isReading = this.push(priv.incomingQueue.shift());
            }
        }
    }, {
        key: '_write',
        value: function _write(chunk, encoding, callback) {
            this.send(chunk.toString());
            callback();
        }
    }, {
        key: 'send',
        value: function send(data) {
            var _this4 = this;

            this[$private].outgoingQueue.push(data);
            (0, _process.nextTick)(function () {
                return processQueue.call(_this4);
            });
        }
    }, {
        key: 'close',
        value: function close() {
            var socket = this[$private].socket;

            socket && socket.end();
        }
    }, {
        key: 'connected',
        get: function get() {
            return this[$private].connected;
        }
    }]);

    return QueuedSocket;
}(_stream.Duplex);

exports.default = QueuedSocket;