'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require('events');

var _process = require('process');

var _dgram = require('dgram');

var _dgram2 = _interopRequireDefault(_dgram);

var _jsLogger = require('js-logger');

var _jsLogger2 = _interopRequireDefault(_jsLogger);

var _Device = require('./Device');

var _Device2 = _interopRequireDefault(_Device);

var _constants = require('./constants');

var _constants2 = _interopRequireDefault(_constants);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * @module
 */

var BROADCAST_ADDR = _constants2.default.BROADCAST_ADDR;
var SENSEME_PORT = _constants2.default.SENSEME_PORT;
var MISSING_THRESHOLD = _constants2.default.MISSING_THRESHOLD;
var DEFAULT_SCAN_INTERVAL = _constants2.default.DEFAULT_SCAN_INTERVAL;


var $private = Symbol();

/**
 * Discover and manage SenseME-enabled devices on the local network.
 */

var SenseMe = function (_EventEmitter) {
    _inherits(SenseMe, _EventEmitter);

    function SenseMe() {
        _classCallCheck(this, SenseMe);

        var _this = _possibleConstructorReturn(this, (SenseMe.__proto__ || Object.getPrototypeOf(SenseMe)).apply(this, arguments));

        _this[$private] = {
            registry: {},
            nameToId: {}
        };
        return _this;
    }

    /**
     * Process one discovered device.  This will handle
     * tracking of devices in case they eventuall disappear,
     * and in the case of a previously unknown device will
     * emit a "founddevice" event.
     * @private
     * @param {Object} opts
     * @param {string} opts.name - The advertised name of the new device
     * @param {string} opts.id   - The MAC address of the new device
     * @param {string} opts.type - The type of the new device, such as `FAN,HAIKU,HSERIES`
     * @param {string} opts.ip   - The IP address of the new device
     * @emits SenseMe#founddevice
     */


    _createClass(SenseMe, [{
        key: '_handleDeviceFound',
        value: function _handleDeviceFound(_ref) {
            var name = _ref.name;
            var id = _ref.id;
            var type = _ref.type;
            var ip = _ref.ip;
            var _$private = this[$private];
            var registry = _$private.registry;
            var nameToId = _$private.nameToId;


            var lastseen = Date.now();
            if (registry[id]) {
                registry[id].lastseen = lastseen;
            } else {
                var messenger = new _events.EventEmitter();
                registry[id] = {
                    lastseen: lastseen,
                    messenger: messenger,
                    device: new _Device2.default({ name: name, id: id, type: type, ip: ip }, messenger)
                };

                nameToId[name] = id;
                registry[id].device.refreshAll();

                this.emit('founddevice', registry[id].device);
            }
        }

        /**
         * Process one incoming message from a remote device.
         * @param {string} name - The name of the remote device
         * @param {string} msg - The actual message
         * @private
         */

    }, {
        key: '_handleMessage',
        value: function _handleMessage(name, msg) {
            var _$private2 = this[$private];
            var nameToId = _$private2.nameToId;
            var _$private2$registry = _$private2.registry;
            _$private2$registry = _$private2$registry === undefined ? {} : _$private2$registry;
            var _$private2$registry$n = _$private2$registry[nameToId[name]];
            _$private2$registry$n = _$private2$registry$n === undefined ? {} : _$private2$registry$n;
            var _$private2$registry$n2 = _$private2$registry$n.messenger;
            var messenger = _$private2$registry$n2 === undefined ? undefined : _$private2$registry$n2;


            if (messenger) {
                messenger.emit('message', msg);
            }
        }

        /**
         * Get a list of all currently known devices.
         * @returns {Array<Device>}
         */

    }, {
        key: 'getAllDevices',
        value: function getAllDevices() {
            var registry = this[$private].registry;

            return Object.keys(registry).map(function (k) {
                return registry[k].device;
            });
        }

        /**
         * Get one discovered device by its ID (usually MAC address).
         * @param {string} id - The ID of the requested device.
         * @returns {Device}
         */

    }, {
        key: 'getDeviceById',
        value: function getDeviceById(id) {
            var registry = this[$private].registry;


            if (id in registry) {
                return registry[id].device;
            }
            return undefined;
        }

        /**
         * Get one discovered device by its name.
         * @param {string} name - The name of the requested device.
         * @returns {Device}
         */

    }, {
        key: 'getDeviceByName',
        value: function getDeviceByName(name) {
            var _$private3 = this[$private];
            var registry = _$private3.registry;
            var nameToId = _$private3.nameToId;


            if (name in nameToId && nameToId[name] in registry) {
                return registry[nameToId[name]].device;
            }
            return undefined;
        }

        /**
         * Begin discovery of all SenseME devices on the local network.
         * Discovery will continue until {@link cancelDiscovery} is called.
         * @param {number} [interval=10000] - How often (in milliseconds) should a discovery request be sent out?
         * @param {number} [missingThreshold=3] - How many discovery requests must a device miss before being considered no longer on the network?
         * @emits SenseMe#lostdevice
         */

    }, {
        key: 'discover',
        value: function discover() {
            var _this2 = this;

            var interval = arguments.length <= 0 || arguments[0] === undefined ? DEFAULT_SCAN_INTERVAL : arguments[0];
            var missingThreshold = arguments.length <= 1 || arguments[1] === undefined ? MISSING_THRESHOLD : arguments[1];
            var registry = this[$private].registry;


            var server = this[$private].server = _dgram2.default.createSocket('udp4');
            server.on('error', function (err) {
                console.log('server error:\n' + err.stack);
                server.close();
            }).on('message', function (msg, _ref2) {
                var ip = _ref2.address;

                if (typeof msg !== 'string') {
                    msg = msg.toString();
                }
                if (!msg.startsWith('(')) return;

                _jsLogger2.default.debug('UDP Incoming: ' + msg);

                var _msg$substr$split = msg.substr(1, msg.length - 2).split(';');

                var _msg$substr$split2 = _slicedToArray(_msg$substr$split, 5);

                var name = _msg$substr$split2[0];
                var namespace = _msg$substr$split2[1];
                var command = _msg$substr$split2[2];
                var id = _msg$substr$split2[3];
                var type = _msg$substr$split2[4];


                if (namespace === 'DEVICE' && command === 'ID') {
                    _this2._handleDeviceFound({ name: name, id: id, type: type, ip: ip });
                } else {
                    _this2._handleMessage(name, msg);
                }
            }).on('listening', function () {
                var address = server.address();
                console.log('server listening ' + address.address + ':' + address.port);
            }).on('close', function () {
                console.log('server shutting down');
            });

            server.bind(SENSEME_PORT);

            var discover = function discover() {
                var client = _dgram2.default.createSocket('udp4');
                client.bind(function () {
                    client.setBroadcast(true);
                    client.send('<ALL;DEVICE;ID;GET>', SENSEME_PORT, BROADCAST_ADDR, function () {
                        client.close();
                    });
                    (0, _process.nextTick)(function () {
                        Object.keys(registry).filter(function (x) {
                            return registry[x].lastseen + missingThreshold * interval < Date.now();
                        }).forEach(function (x) {
                            var dev = registry[x];
                            delete registry[x];
                            _this2.emit('lostdevice', dev.device);
                        });
                    });
                });
            };

            setTimeout(function () {
                discover();
                _this2._discoveryInterval = setInterval(discover, interval);
            }, 100);
        }

        /**
         * Cancel an ongoing discovery session.
         */

    }, {
        key: 'cancelDiscovery',
        value: function cancelDiscovery() {
            clearInterval(this._discoveryInterval);
            this._discoveryInterval = undefined;
            _jsLogger2.default.info("Discovery cancelled");

            this[$private].server.close();
            this[$private].server = undefined;
        }
    }]);

    return SenseMe;
}(_events.EventEmitter);

var single = new SenseMe();

exports.default = single;