'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require('events');

var _observable = require('./lib/observable');

var _observable2 = _interopRequireDefault(_observable);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function getValueAtPath(path, base) {
    return [].concat(path).reduce(function (acc, key) {
        if (acc) return acc[key];
        return undefined;
    }, base);
}

function getParentAtPath(path, base) {
    var pathParts = [].concat(path);
    var lastPart = pathParts.pop();

    return {
        object: pathParts.reduce(function (a, k) {
            return a[k] ? a[k] : a[k] = {};
        }, base),
        valueKey: lastPart
    };
}

var $private = Symbol();

var State = function () {
    function State() {
        _classCallCheck(this, State);

        this[$private] = {
            allListeners: [],
            stateListeners: {},

            stateStore: {}
        };

        if (_observable.hasObservable) {
            Object.assign(this[$private], {
                allObservers: [],
                stateObservers: {}
            });

            this.observeAll = function () {
                var _this = this;

                return new _observable2.default(function (subscriber) {
                    _this[$private].allObservers.push(subscriber);
                    return function () {
                        _this[$private].allObservers.remove(subscriber);
                    };
                });
            };

            this.observe = function (path) {
                var _this2 = this;

                var _getParentAtPath = getParentAtPath(path, this[$private].stateObservers);

                var acc = _getParentAtPath.object;
                var valueKey = _getParentAtPath.valueKey;


                if (acc) {
                    var _ret = function () {
                        if (!acc[valueKey]) {
                            acc[valueKey] = [];
                        }

                        var self = _this2;
                        return {
                            v: _observable2.default.create(function (subscriber) {
                                subscriber.next(self.get(path));
                                acc[valueKey].push(subscriber);
                                return function () {
                                    acc[valueKey].remove(subscriber);
                                };
                            })
                        };
                    }();

                    if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
                }

                return new _observable2.default(function (s) {
                    return s.complete();
                });
            };
        }
    }

    _createClass(State, [{
        key: 'listen',
        value: function listen(path) {
            var _getParentAtPath2 = getParentAtPath(path, this[$private].stateListeners);

            var acc = _getParentAtPath2.object;
            var valueKey = _getParentAtPath2.valueKey;


            var ret = new _events.EventEmitter();
            if (acc) {
                if (!acc[valueKey]) {
                    acc[valueKey] = [];
                }

                ret.emit('change', this.get(path));
                acc[valueKey].push(ret);
            }
            return ret;
        }
    }, {
        key: 'listenAll',
        value: function listenAll() {
            var ret = new _events.EventEmitter();
            this[$private].allListeners.push(ret);
            return ret;
        }
    }, {
        key: 'get',
        value: function get(path) {
            return getValueAtPath(path, this[$private].stateStore);
        }
    }, {
        key: 'set',
        value: function set(path, value) {
            var _$private = this[$private];
            var stateStore = _$private.stateStore;
            var stateObservers = _$private.stateObservers;
            var allObservers = _$private.allObservers;
            var stateListeners = _$private.stateListeners;
            var allListeners = _$private.allListeners;

            var _getParentAtPath3 = getParentAtPath(path, stateStore);

            var acc = _getParentAtPath3.object;
            var valueKey = _getParentAtPath3.valueKey;


            if (acc) {
                if (typeof value !== 'undefined') {
                    value = value.toString();
                }
                acc[valueKey] = value;

                if (_observable.hasObservable) {
                    var observers = getValueAtPath(path, stateObservers);
                    if (observers && observers.length) observers.forEach(function (o) {
                        return o.next(value);
                    });
                    if (allObservers.length) {
                        allObservers.forEach(function (o) {
                            return o.next({ path: path, value: value });
                        });
                    }
                }

                {
                    (function () {
                        var listeners = getValueAtPath(path, stateListeners);
                        if (listeners && listeners.length) {
                            listeners.forEach(function (l) {
                                if (l.listenerCount('change') > 0) {
                                    l.emit('change', value);
                                } else {
                                    // lose that one from the list.
                                    listeners.remove(l);
                                }
                            });
                        }

                        if (allListeners.length) {
                            allListeners.forEach(function (l) {
                                if (l.listenerCount('change') > 0) {
                                    l.emit('change', { path: path, value: value });
                                } else {
                                    // lose that one from the list.
                                    allListeners.remove(l);
                                }
                            });
                        }
                    })();
                }
            }
        }

        // this is like 5/8ths-assed at best.

    }, {
        key: 'delete',
        value: function _delete(path) {
            var _this3 = this;

            var _getParentAtPath4 = getParentAtPath(path, this[$private].stateStore);

            var acc = _getParentAtPath4.object;
            var valueKey = _getParentAtPath4.valueKey;


            if (acc) {
                (function () {
                    var deletedValue = acc[valueKey];
                    delete acc[valueKey];

                    if (_observable.hasObservable) {
                        var observers = getValueAtPath(path, _this3[$private].stateObservers);
                        if (observers && observers.length) observers.forEach(function (o) {
                            return o.next(undefined);
                        });
                    }
                    {
                        if (listeners && listeners.length) {
                            listeners.forEach(function (l) {
                                if (l.listenerCount('change') > 0) {
                                    l.emit('delete', deletedValue);
                                } else {
                                    // lose that one from the list.
                                    listeners.remove(l);
                                }
                            });
                        }

                        if (allListeners.length) {
                            allListeners.forEach(function (l) {
                                if (l.listenerCount('change') > 0) {
                                    l.emit('delete', deletedValue);
                                } else {
                                    // lose that one from the list.
                                    allListeners.remove(l);
                                }
                            });
                        }
                    }
                })();
            }
        }
    }]);

    return State;
}();

;

exports.default = State;