'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.translateHardwareValues = exports.mapForward = exports.mapBack = exports.validateAndMap = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _computePolynomial = require('compute-polynomial');

var _computePolynomial2 = _interopRequireDefault(_computePolynomial);

var _justValues = require('just-values');

var _justValues2 = _interopRequireDefault(_justValues);

var _pathMappingVisitor = require('./walk/pathMappingVisitor');

var _specVisitor = require('./walk/specVisitor');

var _round = require('./util/round');

var _round2 = _interopRequireDefault(_round);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var validators = {
    power: function power(value) {
        return value in { on: 'ON', off: 'OFF' };
    },

    number: function number(value, spec) {
        return typeof value === 'number' && !spec.values || value >= spec.values[0] && value <= spec.values[1];
    },
    enum: function _enum(value, spec) {
        return spec.values && (0, _justValues2.default)(spec.values).includes(value);
    },
    boolean: function boolean(value) {
        return typeof value === 'boolean';
    },
    temperature: function temperature(value) {
        return typeof value === 'number' && value >= 10 && value <= 32.22;
    },
    height: function height(value) {
        return typeof value === 'number';
    },
    string: function string() {
        return true;
    }
};

var forwardMappings = {
    power: function power(value) {
        return { on: 'ON', off: 'OFF' }[value];
    },
    number: function number(value, _ref) {
        var _ref$values = _ref.values;
        _ref$values = _ref$values === undefined ? [] : _ref$values;

        var _ref$values2 = _slicedToArray(_ref$values, 2);

        var _ref$values2$ = _ref$values2[0];
        var min = _ref$values2$ === undefined ? Number.MIN_SAFE_INTEGER : _ref$values2$;
        var _ref$values2$2 = _ref$values2[1];
        var max = _ref$values2$2 === undefined ? Number.MAX_SAFE_INTEGER : _ref$values2$2;
        return Math.min(Math.max(value, min), max);
    },

    enum: function _enum(value, spec) {
        for (var k in spec.values) {
            if (spec.values[k] === value) {
                return k;
            }
        }

        throw new Error('Unable to forward-map ' + value);
    },

    boolean: function boolean(value, spec) {
        return spec.values[Number(!!value)];
    },
    temperature: function temperature(value) {
        // by the time we get here, any necessary F -> C conversion
        // has already happened.  all we need to do is round to 2
        // decimal places, remove the point.
        return value.toFixed(2).replace(/\./, '');
    },

    height: function height(value) {
        // by the time we get here, any necessary m -> ft conversion
        // will have already happened.  all we need to do is roud to
        // 2 decimal places and remove the point.
        return value.toFixed(2).replace(/\./, '');
    },

    string: function string(value) {
        return value.toString();
    }
};

function validateAndMap(path, value) {
    var spec = _specVisitor.specByDataPath[path.join(';')];
    if (spec.type in validators && spec.type in forwardMappings) {
        if (validators[spec.type](value, spec)) {
            return forwardMappings[spec.type](value, spec);
        } else {
            throw new Error('Value ' + value + ' does not match its spec');
        }
    }

    throw new Error("I don't know this type: " + spec.type);
}

function mapForward(path, value) {
    var spec = _specVisitor.specByDataPath[path.join(';')];
    if (spec.type in forwardMappings) {
        return forwardMappings[spec.type](value, spec);
    }

    throw new Error("I don't know this type: " + spec.type);
}

var reverseMappings = {
    number: function number(v) {
        return Number(v);
    },
    string: function string(v) {
        return v;
    },
    boolean: function boolean(value, spec) {
        var idx = spec.values.indexOf(value);
        return idx >= 0 && !!idx;
    },

    enum: function _enum(value, spec) {
        return spec.values[value];
    },

    power: function power(value) {
        return { ON: 'on', OFF: 'off' }[value];
    },

    temperature: function temperature(value) {
        return Number(value.replace(/(\d*)(\d{2})$/, '$1.$2'));
    },

    height: function height(value) {
        return Number(value.replace(/(\d*)(\d{2})$/, '$1.$2'));
    }
};

function mapBack(path, value) {
    var spec = _specVisitor.specByDataPath[path.join(';')];
    if (spec && spec.type in reverseMappings) {
        return reverseMappings[spec.type](value, spec);
    }

    throw new Error("I don't know this type: " + spec.type);
}

function translateHardwareValues(_ref2) {
    var path = _ref2.path;
    var value = _ref2.value;

    var spec = _specVisitor.specByDataPath[path.join(';')];
    if (!spec) return [];

    var results = [];

    var friendlyPath = _pathMappingVisitor.realPathToFriendlyPath[path.join(';')].split(';'),
        currentValue = mapBack(path, value);

    results.push({
        path: [].concat(friendlyPath),
        value: currentValue
    });

    if (spec.computed) {
        spec.computed.map(function (_ref3) {
            var _context;

            var friendlyName = _ref3.friendlyName;
            var forward = _ref3.forward;
            return {
                friendlyName: friendlyName,
                value: (_context = (0, _computePolynomial2.default)(forward, currentValue), _round2.default).call(_context, 2)
            };
        }).forEach(function (_ref4) {
            var friendlyName = _ref4.friendlyName;
            var value = _ref4.value;

            friendlyPath[friendlyPath.length - 1] = friendlyName;

            results.push({
                path: [].concat(friendlyPath),
                value: value
            });
        });
    }

    return results;
}
exports.validateAndMap = validateAndMap;
exports.mapBack = mapBack;
exports.mapForward = mapForward;
exports.translateHardwareValues = translateHardwareValues;