'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.accessorGenerators = exports.default = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _events = require('events');

var _justCompose = require('just-compose');

var _justCompose2 = _interopRequireDefault(_justCompose);

var _computePolynomial = require('compute-polynomial');

var _computePolynomial2 = _interopRequireDefault(_computePolynomial);

var _mapValues = require('../mapValues');

var _observable = require('../observable');

var _observable2 = _interopRequireDefault(_observable);

var _map = require('../observable/map');

var _round = require('../util/round');

var _round2 = _interopRequireDefault(_round);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var accessorGenerators = {};

function accessorVisitor(name, props, node, context) {
    var generatorStore = context.generatorStore;


    if (typeof generatorStore === 'undefined') {
        generatorStore = accessorGenerators;
    }

    if (props.type === 'group') {
        if (!(props.friendlyName in generatorStore)) {
            Object.defineProperty(generatorStore, props.friendlyName, {
                writable: false,
                enumerable: true,
                value: {}
            });
        }
        generatorStore = generatorStore[props.friendlyName];
    } else if (props.type !== undefined) {
        // first, create a home to own these
        generatorStore[props.friendlyName] = {
            value: getDescriptor(props, context)
        };

        if (props.computed && props.computed.length > 0) {
            props.computed.forEach(function (_ref) {
                var friendlyName = _ref.friendlyName;
                var forward = _ref.forward;
                var reverse = _ref.reverse;

                generatorStore[friendlyName] = Object.defineProperties({}, {
                    isComputed: { value: true },
                    baseName: { value: props.friendlyName },
                    func: { value: getComputedDescriptor({ props: props, forward: forward, reverse: reverse, context: context }) }
                });
            });
        }

        generatorStore = generatorStore[props.friendlyName];
    }

    return _extends({}, context, { generatorStore: generatorStore });
}

function getDescriptor(props, context) {
    var _context$calculatedPa = context.calculatedPaths;
    var getPath = _context$calculatedPa.getPath;
    var setPath = _context$calculatedPa.setPath;
    var refreshPath = _context$calculatedPa.refreshPath;
    var parentPath = context.parentPath;


    return function (accessors) {
        var value = {
            enumerable: true,
            get: function get() {
                return accessors.get(getPath);
            }
        };

        if (props.settable) {
            value.set = function (value) {
                var mapped = (0, _mapValues.validateAndMap)(getPath, value);
                accessors.set(setPath, mapped);
            };
        }

        var functions = {
            refresh: {
                writable: false,
                enumerable: true,
                value: function value() {
                    return accessors.refresh(refreshPath);
                }
            },
            listen: {
                writable: false,
                enumerable: true,
                value: function value() {
                    return accessors.listen(getPath);
                }
            }
        };

        if (_observable.hasObservable) {
            functions.observe = {
                writable: false,
                enumerable: true,
                value: function value() {
                    return accessors.observe(getPath);
                }
            };
        }

        return { functions: functions, value: value };
    };
}

function getComputedDescriptor(_ref2) {
    var props = _ref2.props;
    var forward = _ref2.forward;
    var reverse = _ref2.reverse;
    var context = _ref2.context;
    var getPath = context.calculatedPaths.getPath;


    return function (base) {
        var value = {
            enumerable: true,
            get: function get() {
                var _context;

                return (_context = (0, _computePolynomial2.default)(forward, base.value), _round2.default).call(_context, 2);
            }
        };

        if (props.settable) {
            value.set = function (value) {
                base.value = (0, _computePolynomial2.default)(reverse, value);
            };
        }

        var functions = {
            refresh: {
                writable: false,
                enumerable: true,
                value: function value() {
                    return base.refresh();
                }
            },
            listen: {
                writable: false,
                enumerable: true,
                value: function value() {
                    var ret = new _events.EventEmitter();
                    base.listen().on('change', (0, _justCompose2.default)(function (x) {
                        var _context2;

                        return (_context2 = (0, _computePolynomial2.default)(forward, (0, _mapValues.mapBack)(getPath, x)), _round2.default).call(_context2, 2);
                    }, function (x) {
                        return ret.emit('change', x);
                    }));
                    return ret;
                }
            }
        };

        if (_observable.hasObservable) {
            functions.observe = {
                writable: false,
                enumerable: true,
                value: function value() {
                    var _context3;

                    return (_context3 = base.observe(), _map.method).call(_context3, function (x) {
                        var _context4;

                        return x ? (_context4 = (0, _computePolynomial2.default)(forward, (0, _mapValues.mapBack)(getPath, x)), _round2.default).call(_context4, 2) : x;
                    });
                }
            };
        }

        return { functions: functions, value: value };
    };
}

exports.default = accessorVisitor;
exports.accessorGenerators = accessorGenerators;